package org.fuseri.moduleexercise.question;

import jakarta.persistence.EntityNotFoundException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.PositiveOrZero;
import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.fuseri.model.dto.exercise.QuestionUpdateDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

/**
 * Represent a REST API controller for questions
 * Handle HTTP requests related to questions
 */
@RestController
@RequestMapping("/questions")
public class QuestionController {

    private final QuestionFacade questionFacade;

    /**
     * Constructor for QuestionController
     *
     * @param questionFacade the service responsible for handling question-related logic
     */
    @Autowired
    public QuestionController(QuestionFacade questionFacade) {
        this.questionFacade = questionFacade;
    }

    /**
     * Find a question by ID.
     *
     * @param id the ID of the question to find
     * @return a QuestionDto object representing the found question
     */
    @GetMapping("/{id}")
    public QuestionDto find(@NotBlank @PathVariable String id) {
        return questionFacade.find(id);
    }

    /**
     * Find questions by exercise ID and return them in a paginated format
     *
     * @param exerciseId the ID of the exercise to find questions for
     * @param page       the page number of the questions to retrieve
     * @return a Result object containing a list of QuestionDto objects and pagination information
     */
    @GetMapping("/exercise/{exercise-id}")
    public Result<QuestionDto> findByExerciseId(@NotBlank @PathVariable("exercise-id") String exerciseId,
                                                @PositiveOrZero @RequestParam int page) {
        return questionFacade.findByExerciseId(exerciseId, page);
    }

    /**
     * Add a new question to an exercise
     *
     * @param dto a QuestionCreateDto object representing the new question to add
     * @return a QuestionDto object representing the added question
     * @throws ResponseStatusException if the exercise with exerciseId from QuestionCreateDto does not exist
     */
    @PostMapping
    public QuestionDto addQuestion(@Valid @RequestBody QuestionCreateDto dto) {
        try {
            return questionFacade.create(dto);
        } catch (EntityNotFoundException e) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND,
                    e.getMessage());
        }
    }

    /**
     * Update question
     *
     * @param dto a QuestionDto object representing the updated question with correct id
     * @return a QuestionUpdateDto object representing the updated question
     * @throws ResponseStatusException if the question with id doesn't exist or its exercise doesn't exist
     */
    @PutMapping("/{id}")
    public QuestionDto updateQuestion(@NotBlank @PathVariable String id, @Valid @RequestBody QuestionUpdateDto dto) {
        try {
            return questionFacade.update(id, dto);
        } catch (IllegalArgumentException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        } catch (EntityNotFoundException e) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, e.getMessage());
        }
    }

    /**
     * Add a new question to an exercise
     *
     * @param id of question to delete
     */
    @DeleteMapping("/{id}")
    public void deleteQuestion(@NotBlank @PathVariable String id) {
        questionFacade.delete(id);
    }
}